package net.stefankrause.jbpm;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.ClockType;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SessionConfiguration;
import org.drools.base.MapGlobalResolver;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.conf.ClockTypeOption;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public class ProcessRunner {

	private PoolingDataSource ds1;
	protected StatefulKnowledgeSession ksession;
	protected KnowledgeRuntimeLogger logger;
	protected KnowledgeBase knowledgeBase;

	private final String processResourceName;
	private EntityManagerFactory emf;
	private final Integer knowledgeSessionId;
	private final boolean createLogger;

	public ProcessRunner(Integer knowledgeSessionId, String processResourceName, boolean createLogger) {
		this.knowledgeSessionId = knowledgeSessionId;
		this.processResourceName = processResourceName;
		this.createLogger = createLogger;
		setupDS();
		try {
			knowledgeBase = readKnowledgeBase();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		setupEntityManagers();
		setupSession(createLogger);
	}
	
	public void shutDown() {
		if (logger!=null) logger.close();
		ksession.dispose();
		ds1.close();
	}
	
	private void setupDS() {
		ds1 = new PoolingDataSource();
		ds1.setMaxPoolSize(5);
		ds1.setAllowLocalTransactions(true);
		ds1.setUniqueName("jdbc/processInstanceDS");
		ds1.setClassName("org.h2.jdbcx.JdbcDataSource");
		ds1.getDriverProperties().setProperty("URL", "jdbc:h2:~/JBPM52Bugs_JBPM");
		ds1.init();
	}

	private KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource(processResourceName),
				ResourceType.BPMN2);
		return kbuilder.newKnowledgeBase();
	}
	
	private void setupEntityManagers() {
        emf = Persistence.createEntityManagerFactory( "org.jbpm.persistence.jpa" );
	}
	
	private void setupSession(boolean createLogger) {
        Environment env = KnowledgeBaseFactory.newEnvironment();
        env.set( EnvironmentName.ENTITY_MANAGER_FACTORY, emf );
        env.set( EnvironmentName.GLOBALS, new MapGlobalResolver() );
		env.set(EnvironmentName.TRANSACTION_MANAGER, TransactionManagerServices.getTransactionManager());

		SessionConfiguration conf = new SessionConfiguration();
//	    conf.setOption( ClockTypeOption.get( ClockType.PSEUDO_CLOCK.getId() ) ); 

		if (knowledgeSessionId==null) {
			System.out.println("Creating a new session");
			ksession = JPAKnowledgeService.newStatefulKnowledgeSession(knowledgeBase, conf, env);
		} else {
			System.out.println("Restoring session "+knowledgeSessionId);
			ksession = JPAKnowledgeService.loadStatefulKnowledgeSession(knowledgeSessionId, knowledgeBase, conf, env);
		}
		System.out.println("SESSION ID =" +ksession.getId());
		
		if (createLogger) {
			logger = KnowledgeRuntimeLoggerFactory.newConsoleLogger(ksession);
		}
	}
	
	public StatefulKnowledgeSession getKsession() {
		return ksession;
	}

	public KnowledgeBase getKnowledgeBase() {
		return knowledgeBase;
	}	
}